/*
  groupcheck.c
  written by Olaf Titz, July 1997 as part of c-nocem.
  Public domain.

  Reads pattern file (e.g. GUP format) from specified file.
  The reads lines from stdin: <mid> group group...
  Prints IDs for which at least one group matches the pattern file.

*/
static char *RCSID="$Id: groupcheck.c,v 1.3 1998/12/14 18:58:22 olaf Exp $";

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#define MAXLINE 10000
#define MAXNPAT 10000

extern int wildmat(const char *text, const char *pattern); /* from libinn */

static void usage(const char *p)
{
    fprintf(stderr, "usage: %s patfile\n", p);
    exit(1);
    /* NOTREACHED */
}

static char *ptns[MAXNPAT];
static int nptns=0;

static void read_pat(const char *f)
{
    struct stat st;
    char *p;
    int a=open(f, O_RDONLY);
    if (a<0) {
	perror("open"); exit(1);
    }
    if (fstat(a, &st)<0) {
	perror("fstat"); exit(1);
    }
    if (!(p=malloc(st.st_size+1))) {
	perror("malloc"); exit(1);
    }
    memset(p, 0, st.st_size+1);
    if (read(a, p, st.st_size)!=st.st_size) {
	perror("read/short"); exit(1);
    }
    close(a);
    if (st.st_size>0) {
	(void) strtok(p, " \t\n,");
	do {
	    ptns[nptns++]=p;
	    if (nptns>=MAXNPAT) {
		fprintf(stderr, "Too many patterns in %s\n", f);
		exit(1);
	    }
	} while ((p=strtok(NULL, " \t\n,")));
    }
    --nptns;
}

static int do_match(char *g)
{
    char *p;
    int i, m;

    for (i=nptns; i>=0; --i) {
	p=ptns[i];
	if (*p=='!') {
	    ++p; m=0;
	} else {
	    m=1;
	}
	if (wildmat(g, p))
	    return m;
    }
    return 0;
}

static void do_line(char *lin)
{
    char *p;

    /* skip ID */
    (void) strtok(lin, " \t\n");
    while ((p=strtok(NULL, "\t \n"))) {
	if (do_match(p)) {
	    fputs(lin, stdout);
	    fputc('\n', stdout);
	    return;
	}
    }
}

int main(int argc, char *argv[])
{
    char buf[MAXLINE];
    int c0, c1=1;

    if (argc!=2)
	usage(argv[0]);
    read_pat(argv[1]);

    while (fgets(buf, sizeof(buf), stdin)) {
	c0=c1;
	c1=(buf[strlen(buf)-1]=='\n');
	if (!c0)
	    continue; /* skip continuation bogosities */
	if (!c1) {
	    fprintf(stderr, "stdin: long line\n");
	    continue;
	}
	do_line(buf);
    }
    exit(0);
}
